@@ -63,19 +63,29 @@ import logging
 from functools import cached_property
 
 import warnings
+
 with warnings.catch_warnings():
     warnings.filterwarnings('ignore', message="Using slow pure-python SequenceMatcher")
     # suppress that warning, it's confusing at best here, we don't need fast sequence matching
     # and the installation (on windows) some effort
-    from fuzzywuzzy import fuzz
+    from thefuzz import fuzz
 
 logging.basicConfig(level=logging.INFO, style='{', format="{module: <8} {levelname: >10} \t{message}")
 
-
 TESTING_LOOKUP = {'2020': [['2020-02-19', '2020-02-20', '2020-02-21'],
                            ['2020-02-26', '2020-02-27', '2020-02-28']],
                   '2021': [['2021-03-12', '2021-03-13', '2021-03-14']]}
 
+# TODO: remove; temporary patch to support sprint qualifying
+SPRINT_QUALIFYING_LOOKUP = {
+    10: {'Practice 1': '2021-07-16', 'Qualifying': '2021-07-16',
+         'Practice 2': '2021-07-17', 'Sprint Qualifying': '2021-07-17',
+         'Race': '2021-07-18'},
+    14: {'Practice 1': '2021-09-10', 'Qualifying': '2021-09-10',
+         'Practice 2': '2021-09-11', 'Sprint Qualifying': '2021-09-11',
+         'Race': '2021-09-12'},
+}
+
 D_LOOKUP = [[44, 'HAM', 'Mercedes'], [77, 'BOT', 'Mercedes'],
             [55, 'SAI', 'Ferrari'], [16, 'LEC', 'Ferrari'],
             [33, 'VER', 'Red Bull'], [11, 'PER', 'Red Bull'],
@@ -125,8 +135,11 @@ def get_session(year, gp, event=None):
                                Pass 'testing' to fetch Barcelona winter
                                tests.
 
-        event (=None): may be 'FP1', 'FP2', 'FP3', 'Q' or 'R', if not
+        event (=None): may be 'FP1', 'FP2', 'FP3', 'Q', 'SQ' or 'R', if not
                        specified you get the full :class:`Weekend`.
+                       'SQ' stands for Sprint Qualifying which is only
+                       available in the 2021 season. Note that 'FP3' does
+                       not exist on these race weekends.
                        If gp is 'testing' event is the test day (1 to 6)
 
     Returns:
@@ -145,6 +158,8 @@ def get_session(year, gp, event=None):
         return Session(weekend, 'Race')
     if event == 'Q':
         return Session(weekend, 'Qualifying')
+    if event == 'SQ':  # TODO: remove; temporary patch to support sprint qualifying
+        return Session(weekend, 'Sprint Qualifying')
     if event == 'FP3':
         return Session(weekend, 'Practice 3')
     if event == 'FP2':
@@ -176,7 +191,8 @@ def get_round(year, match):
 
     races = ergast.fetch_season(year)
     to_match = [build_string(block) for block in races]
-    ratios = np.array([fuzz.partial_ratio(match, ref) for ref in to_match])
+    ratios = np.array([fuzz.partial_ratio(match.casefold(), ref.casefold())
+                       for ref in to_match])
 
     return int(races[np.argmax(ratios)]['round'])
 
@@ -284,8 +300,8 @@ class Telemetry(pd.DataFrame):
         'Y': {'type': 'continuous', 'missing': 'quadratic'},
         'Z': {'type': 'continuous', 'missing': 'quadratic'},
         'Status': {'type': 'discrete'},
-        'Speed': {'type': 'continuous', 'missing': 'linear'},     # linear is often required as quadratic overshoots
-        'RPM': {'type': 'continuous', 'missing': 'linear'},       # on sudden changes like sudden pedal application,
+        'Speed': {'type': 'continuous', 'missing': 'linear'},  # linear is often required as quadratic overshoots
+        'RPM': {'type': 'continuous', 'missing': 'linear'},  # on sudden changes like sudden pedal application,
         'Throttle': {'type': 'continuous', 'missing': 'linear'},  # braking, ...)
         'Brake': {'type': 'continuous', 'missing': 'linear'},
         'DRS': {'type': 'discrete'},
@@ -638,7 +654,7 @@ class Telemetry(pd.DataFrame):
                     warnings.warn("Interpolation not possible for telemetry "
                                   "channel because dtype is 'object'")
                 missing = self._CHANNELS[ch]['missing']
-                ret.loc[:, ch] = ret.loc[:, ch]\
+                ret.loc[:, ch] = ret.loc[:, ch] \
                     .interpolate(method=missing, limit_direction='both', fill_value='extrapolate')
 
             elif sig_type == 'discrete':
@@ -712,7 +728,7 @@ class Telemetry(pd.DataFrame):
             {'DifferentialDistance': self.calculate_differential_distance()}
         )
         if 'DifferentialDistance' in self.columns:
-            return self.drop(labels='DifferentialDistance', axis=1)\
+            return self.drop(labels='DifferentialDistance', axis=1) \
                 .join(new_dif_dist, how='outer')
 
         return self.join(new_dif_dist, how='outer')
@@ -796,7 +812,7 @@ class Telemetry(pd.DataFrame):
         """
         if 'DriverAhead' in self.columns and 'DistanceToDriverAhead' in self.columns:
             if drop_existing:
-                d = self.drop(labels='DriverAhead', axis=1)\
+                d = self.drop(labels='DriverAhead', axis=1) \
                     .drop(labels='DistanceToDriverAhead', axis=1)
             else:
                 return self
@@ -898,7 +914,7 @@ class Telemetry(pd.DataFrame):
 
             # first slice by lap and calculate distance, so that distance is zero at finish line
             drv_tel = self.session.car_data[drv].slice_by_lap(relevant_laps).add_distance() \
-                .loc[:, ('SessionTime', 'Distance')].rename(columns={'Distance': drv})
+                          .loc[:, ('SessionTime', 'Distance')].rename(columns={'Distance': drv})
 
             # now slice again by time to only get the relevant time frame
             drv_tel = drv_tel.slice_by_time(t_start, t_end)
@@ -948,6 +964,7 @@ class Weekend:
         race = weekend.get_race() # R Session
 
     """
+
     def __init__(self, year, gp):
         self.year = year
         self.gp = gp
@@ -1027,6 +1044,7 @@ class Session:
 
     .. note:: For full functionality lap and telemetry data need to be loaded.
     """
+
     def __init__(self, weekend, session_name):
         self.weekend = weekend
         """:class:`.Weekend`: Reference to the associated weekend object."""
@@ -1037,6 +1055,15 @@ class Session:
         self.api_path = api.make_path(self.weekend.name,
                                       self.weekend.date,
                                       self.name, self.date)
+
+        self.results = list()
+        """list: Race result with driver information."""
+
+        self.session_status = dict()
+        """pd.Dataframe: Session status data as returned by
+        `fastf1.api.session_status_data()` as dataframe. Available after
+        calling `Session.load_laps()`"""
+
         if not self.weekend.is_testing():
             # The Ergast API can provide some general information about weekends, drivers, ...
             # See ergast.com
@@ -1105,6 +1132,11 @@ class Session:
                 day_index = int(self.name[-1]) - 1
                 date = TESTING_LOOKUP[year][0][day_index]
 
+        # TODO: remove; temporary patch to support sprint qualifying
+        elif ((self.weekend.year == 2021) and
+              (self.weekend.gp in SPRINT_QUALIFYING_LOOKUP.keys())):
+            date = SPRINT_QUALIFYING_LOOKUP[self.weekend.gp][self.name]
+
         elif self.name in ('Qualifying', 'Practice 3'):
             # Assuming that quali was one day before race which is not always correct
             # TODO Should check if also formula1 makes this assumption
@@ -1132,6 +1164,23 @@ class Session:
         Downloading and parsing of the data takes a considerable amount of time. Therefore, it is highly recommended
         to enable caching so that most of the data processing needs to be done only once.
 
+        .. note:: **Drivers crashing and retiring**
+
+            *During a session:*
+            An additional last lap is added for a driver if the last timed
+            lap of a driver is not an inlap and the session is aborted next.
+            The `Time` for when this lap was "set" will be set to the time at
+            which the session was aborted.
+
+            *First lap in a race:*
+            A single lap with minimal information will be added in race
+            sessions if a driver does not complete at least one timed lap.
+            The `LapStartTime` for this lap will be set to the start time
+            of the session as with all other laps in a race. The `Time` at
+            which this lap was "set" will be set to the time at which the
+            first driver completes their first lap.
+
+
         .. note:: Absolute time is not super accurate. The moment a lap
             is logged is not always the same and there will be some
             jitter. At the moment lap time reference is synchronised
@@ -1171,27 +1220,59 @@ class Session:
         useful = app_data[['Driver', 'Time', 'Compound', 'TotalLaps', 'New']]
         useful = useful[~useful['Compound'].isnull()]
 
-        self.drivers = list(data['Driver'].unique())
+        # get list of drivers
+        try:
+            driver_info = api.driver_info(self.api_path, livedata=livedata)
+        except Exception as exc:
+            logging.warning("Failed to load driver list")
+            logging.debug("Exception while loading driver list", exc_info=exc)
+            driver_info = {}
+        if driver_info:
+            self.drivers = list(driver_info.keys())
+        else:
+            # fallback, misses drivers who crashed on their first lap
+            self.drivers = list(data['Driver'].unique())
+            if "" in self.drivers:
+                self.drivers.remove("")
 
         if not self.drivers:
             raise NoLapDataError
 
-        # check when a session was started; for a race this indicates the start of the race
+        # check when a session was started; for a race this indicates the
+        # start of the race
         session_status = api.session_status_data(self.api_path,
                                                  livedata=livedata)
         for i in range(len(session_status['Status'])):
             if session_status['Status'][i] == 'Started':
                 self.session_start_time = session_status['Time'][i]
                 break
+        self.session_status = pd.DataFrame(session_status)
 
         for i, driver in enumerate(self.drivers):
             d1 = data[data['Driver'] == driver]
             d2 = useful[useful['Driver'] == driver]
+            only_one_lap = False
 
-            if len(d2) == 0:
-                continue  # no data for this driver; skip
+            if (not len(d1)) or (not len(d2)):
+                # add data for drivers who crashed on the very first lap
+                # as a downside, this potentially adds a nonexistent lap for
+                # drivers who could not start the race
+                if self.name == 'Race':
+                    only_one_lap = True
+                    result = d1.copy()
+                    result['Driver'] = [driver, ]
+                    result['NumberOfLaps'] = 0
+                    result['NumberOfPitStops'] = 0
+                    result['Time'] = data['Time'].min()
+                    if len(d2):
+                        result['Compound'] = d2['Compound'].iloc[0]
+                        result['TotalLaps'] = d2['TotalLaps'].iloc[0]
+                        result['New'] = d2['New'].iloc[0]
+                else:
+                    continue  # no data for this driver; skip
 
-            result = pd.merge_asof(d1, d2, on='Time', by='Driver')
+            else:
+                result = pd.merge_asof(d1, d2, on='Time', by='Driver')
 
             # calculate lap start time by setting it to the 'Time' of the previous lap
             laps_start_time = list(result['Time'])[:-1]
@@ -1200,11 +1281,51 @@ class Session:
                 laps_start_time.insert(0, self.session_start_time)
             else:
                 laps_start_time.insert(0, pd.NaT)
-            result.loc[:, 'LapStartTime'] = pd.Series(laps_start_time, dtype='timedelta64[ns]')
+            result.loc[:, 'LapStartTime'] = pd.Series(
+                laps_start_time, dtype='timedelta64[ns]'
+            )
 
+            # set missing lap start times to pit out time where possible
+            mask = pd.isna(result['LapStartTime']) & (~pd.isna(result['PitOutTime']))
+            result.loc[mask, 'LapStartTime'] = result.loc[mask, 'PitOutTime']
+
+            # create total laps counter
             for npit in result['NumberOfPitStops'].unique():
                 sel = result['NumberOfPitStops'] == npit
                 result.loc[sel, 'TotalLaps'] += np.arange(0, sel.sum()) + 1
+
+            # check if there is another lap during which the session was aborted
+            # if yes, add as much data as possible for it
+            # set the time of abort as lap end time given that there is no
+            # accurate time available
+            if pd.isna(result['PitInTime'].iloc[-1]) and not only_one_lap:
+                if not pd.isna(result['Time'].iloc[-1]):
+                    next_status = self.session_status[
+                        self.session_status['Time'] > result['Time'].iloc[-1]
+                    ].iloc[0]
+                else:
+                    next_status = self.session_status[
+                        self.session_status['Time']
+                        > result['LapStartTime'].iloc[-1]
+                    ].iloc[0]
+
+                if next_status['Status'] == 'Aborted':
+                    new_last = pd.DataFrame({
+                        'LapStartTime': [result['Time'].iloc[-1]],
+                        'Time': [next_status['Time']],
+                        'Driver': [result['Driver'].iloc[-1]],
+                        'NumberOfLaps': [result['NumberOfLaps'].iloc[-1] + 1],
+                        'NumberOfPitStops':
+                            [result['NumberOfPitStops'].iloc[-1]],
+                        'Compound': [result['Compound'].iloc[-1]],
+                        'TotalLaps': [result['TotalLaps'].iloc[-1] + 1],
+                        'New': [result['New'].iloc[-1]],
+                    })
+                    if not only_one_lap:
+                        result = result.append(new_last).reset_index(drop=True)
+                    else:
+                        result = new_last
+
             # check if df is defined already before concat (vars is a builtin function)
             df = result if 'df' not in vars() else pd.concat([df, result], sort=False)  # noqa: F821
 
@@ -1215,9 +1336,11 @@ class Session:
                              'NumberOfLaps': 'LapNumber',
                              'New': 'FreshTyre'}, inplace=True)
         laps['Stint'] += 1  # counting stints from 1
-        t_map = {r['number']: r['Constructor']['name'] for r in self.results}
+        t_map = {r.get('number', ''): r.get('Constructor', {}).get('name', '')
+                 for r in self.results}
         laps['Team'] = laps['DriverNumber'].map(t_map)
-        d_map = {r['number']: r['Driver']['code'] for r in self.results}
+        d_map = {r.get('number', ''): r.get('Driver', {}).get('code', '')
+                 for r in self.results}
         laps['Driver'] = laps['DriverNumber'].map(d_map)
         # add track status data
         ts_data = api.track_status_data(self.api_path, livedata=livedata)
@@ -1395,7 +1518,7 @@ class Session:
         """
         if type(identifier) is str:
             for info in self.results:
-                if info['Driver']['code'] == identifier:
+                if info.get('Driver', {}).get('code') == identifier:
                     return Driver(self, info)
 
         return None
@@ -1648,17 +1771,29 @@ class Laps(pd.DataFrame):
         Returns:
             pandas.DataFrame
 
+        .. testsetup::
+
+                >>> fastf1.Cache.enable_cache('doc_cache')
+
         Example::
 
+            >>> session = fastf1.get_session(2019, 'Monza', 'Q')
+            >>> laps = session.load_laps()
             >>> weather_data = laps.get_weather_data()
             >>> print(weather_data)
                                  Time AirTemp Humidity  ... TrackTemp WindDirection WindSpeed
-            16 0 days 00:16:20.305000    21.9     55.8  ...      35.5           119       0.2
-            17 0 days 00:17:20.301000    22.0     54.6  ...      35.9           113       0.1
+            20 0 days 00:20:14.613000    22.5     52.0  ...      35.8           212       2.0
+            21 0 days 00:21:15.001000    22.5     52.2  ...      36.1           207       2.7
+            23 0 days 00:23:14.854000    22.7     52.5  ...      37.4           210       2.3
+            24 0 days 00:24:14.430000    23.2     51.5  ...      37.4           207       3.2
+            26 0 days 00:26:14.315000    23.6     50.2  ...      37.2           238       1.8
             ..                    ...     ...      ...  ...       ...           ...       ...
-            75 0 days 01:15:20.525000    22.3     53.5  ...      36.9            76       0.6
-            76 0 days 01:16:20.534000    22.2     53.1  ...      37.1            81       0.6
-            [453 rows x 25 columns]
+            23 0 days 00:23:14.854000    22.7     52.5  ...      37.4           210       2.3
+            24 0 days 00:24:14.430000    23.2     51.5  ...      37.4           207       3.2
+            34 0 days 00:34:14.385000    23.0     51.7  ...      37.7           272       0.8
+            36 0 days 00:36:14.426000    23.0     51.1  ...      38.3           192       0.9
+            37 0 days 00:37:14.391000    23.3     50.0  ...      38.7           213       0.9
+            [269 rows x 8 columns]
 
         Joining weather data with lap timing data::
 
@@ -1667,15 +1802,21 @@ class Laps(pd.DataFrame):
             >>> weather_data = weather_data.reset_index(drop=True)
 
             >>> # exclude the 'Time' column from weather data when joining
-            >>> joined = pandas.concat([laps, weather_data.loc[:, ~(weather_data.columns == 'Time')]], axis=1)
+            >>> joined = pd.concat([laps, weather_data.loc[:, ~(weather_data.columns == 'Time')]], axis=1)
             >>> print(joined)
                                   Time DriverNumber  ... WindDirection  WindSpeed
-            0   0 days 00:16:58.221000            3  ...           119        0.2
-            1   0 days 00:19:32.354000            3  ...           113        0.1
+            0   0 days 00:20:55.333000            5  ...           212        2.0
+            1   0 days 00:22:16.210000            5  ...           207        2.7
+            2   0 days 00:24:01.233000            5  ...           210        2.3
+            3   0 days 00:25:21.611000            5  ...           207        3.2
+            4   0 days 00:27:12.045000            5  ...           238        1.8
             ..                     ...          ...  ...           ...        ...
-            451 0 days 01:16:01.722000           99  ...            76        0.6
-            452 0 days 01:17:35.072000           99  ...            81        0.6
-            [453 rows x 32 columns]
+            264 0 days 00:24:02.127000           88  ...           210        2.3
+            265 0 days 00:33:31.080000           88  ...           207        3.2
+            266 0 days 00:35:15.794000           88  ...           272        0.8
+            267 0 days 00:36:38.150000           88  ...           192        0.9
+            268 0 days 00:38:37.508000           88  ...           213        0.9
+            [269 rows x 32 columns]
         """
         wd = [lap.get_weather_data() for _, lap in self.iterrows()]
         if wd:
@@ -1979,21 +2120,27 @@ class Lap(pd.Series):
         Returns:
             pandas.Series
 
+        .. testsetup::
+
+            >>> fastf1.Cache.enable_cache('doc_cache')
+
         Example::
 
+            >>> session = fastf1.get_session(2019, 'Monza', 'Q')
+            >>> laps = session.load_laps()
             >>> lap = laps.pick_fastest()
             >>> lap['LapStartTime']
-            Timedelta('0 days 01:10:29.730000')
+            Timedelta('0 days 01:09:55.561000')
             >>> lap.get_weather_data()
-            Time             0 days 01:11:20.525000
-            AirTemp                            22.1
-            Humidity                           53.5
-            Pressure                         1003.5
+            Time             0 days 01:10:15.292000
+            AirTemp                            23.0
+            Humidity                           51.9
+            Pressure                          992.4
             Rainfall                           True
-            TrackTemp                          36.9
-            WindDirection                        76
-            WindSpeed                           0.6
-            Name: 71, dtype: object
+            TrackTemp                          37.8
+            WindDirection                       166
+            WindSpeed                           0.8
+            Name: 70, dtype: object
         """
         # get first value within the duration of the lap
         mask = ((self.session.weather_data['Time'] >= self['LapStartTime']) &
@@ -2019,6 +2166,7 @@ class Driver:
 
     .. note:: Driver data is only available if the Ergast api lookup did not fail.
     """
+
     def __init__(self, session, info):
         self.session = session
         self.info = info
@@ -2060,6 +2208,7 @@ class Driver:
 
 class NoLapDataError(Exception):
     """Raised if the API request does not fail but there is no usable data after processing the result."""
+
     def __init__(self, *args):
         super(NoLapDataError, self).__init__("Failed to load session because the API did not provide any usable data.")
 
